'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

var _typeof = typeof Symbol === "function" && typeof Symbol.iterator === "symbol" ? function (obj) { return typeof obj; } : function (obj) { return obj && typeof Symbol === "function" && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj; };

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

/**
* Author: Chrioyles
* Date: 06.09.2018
**/

var EZStore = function () {
  /**
  * PUBLIC FUNCTIONS
  **/

  /**
  * Create the store while filling it with data
  **/
  function EZStore(storeObject) {
    _classCallCheck(this, EZStore);

    if (!((typeof storeObject === 'undefined' ? 'undefined' : _typeof(storeObject)) === 'object')) {
      console.warn('Error in module EZStore. Value passed to constructor has to be' + 'typeof Object.');
    };
    this._listeners = [];
    this._data = _extends({}, storeObject);
    this._subscribeIndex = 0;
  }

  _createClass(EZStore, [{
    key: 'set',


    /**
    * Set a value to a key in the store. Key must match a key in the _data Object.
    **/
    value: function set(key, value) {
      if (!(typeof key === 'string') || !this._data.hasOwnProperty(key)) {
        console.warn('Error in module EZStore. ' + 'The key passed to EZStore.set() is not found in the store');
      }
      if (value === this._data[key]) return;
      this._data[key] = Object.assign({}, { value: value }).value;
      this._dispatchChange(this._listeningOn(key));
    }
  }, {
    key: 'get',


    /**
    * Get a value of a key in the store. Key must match a key in the _data Object.
    **/
    value: function get(key) {
      if (!(typeof key === 'string') || !this._data.hasOwnProperty(key)) {
        console.warn('Error in module EZStore. ' + 'The key passed to EZStore.get() is not found in the store');
        return;
      };
      return Object.assign({}, { data: this._data[key] }).data;
    }
  }, {
    key: 'subscribe',


    /**
    * Subscribe to a key in the store. Triggers a change when the value of the key
    * changes
    **/
    value: function subscribe() {
      for (var _len = arguments.length, parameter = Array(_len), _key = 0; _key < _len; _key++) {
        parameter[_key] = arguments[_key];
      }

      var functionsInParameter = parameter.filter(function (parameter) {
        return typeof parameter === 'function';
      });
      var cb = functionsInParameter.length > 0 ? functionsInParameter[0] : null;
      if (!cb) {
        console.warn('Error in module EZStore. ' + 'Subscribe function did not receive a callback function');
        return -1;
      };

      var stringsInParameter = parameter.filter(function (parameter) {
        return typeof parameter === 'string';
      });
      var key = stringsInParameter.length > 0 ? stringsInParameter[0] : null;
      if (!key) {
        console.warn('Error in module EZStore. Subscribe function did not receive a key');
        return -1;
      };

      var listener = {};
      listener[key] = cb;
      listener.index = this._subscribeIndex++;
      this._listeners.push(listener);
      return listener.index;
    }
  }, {
    key: 'unsubscribe',
    value: function unsubscribe(index) {
      this._listeners = this._listeners.filter(function (listener) {
        return listener.index !== index;
      });
    }

    /**
    * PRIVATE FUNCTIONS
    **/

    /**
    * Filters out all subscriber function to the key specified as a parameter.
    **/

  }, {
    key: '_listeningOn',
    value: function _listeningOn(key) {
      if (!(typeof key === 'string')) return;
      return this._listeners.filter(function (listener) {
        return listener.hasOwnProperty(key);
      });
    }
  }, {
    key: '_dispatchChange',


    /**
    * Calls all functions of the listeners specified in the parameters.
    **/
    value: function _dispatchChange(listeners) {
      var that = this;
      listeners.forEach(function (listener) {
        var keyName = Object.keys(listener)[0];
        if (keyName) {
          listener[keyName](that._data[keyName]);
        }
      });
    }
  }]);

  return EZStore;
}();

;

exports.default = EZStore;